@@ -171,7 +171,7 @@ gedit_prefs_manager_init (void)
 			g_warning (_("Cannot initialize preferences manager."));
 			return FALSE;
 		}
-		
+
 		gedit_prefs_manager = g_new0 (GeditPrefsManager, 1);
 
 		gedit_prefs_manager->gconf_client = gconf_client;
@@ -210,7 +210,7 @@ gedit_prefs_manager_get_bool (const gchar* key, gboolean def)
 						   key,
 						   def,
 						   NULL);
-}	
+}
 
 static gint 
 gedit_prefs_manager_get_int (const gchar* key, gint def)
@@ -294,93 +294,6 @@ gedit_prefs_manager_key_is_writable (const gchar* key)
 	return gconf_client_key_is_writable (gedit_prefs_manager->gconf_client, key, NULL);
 }
 
-static GdkColor
-gconf_client_get_color_with_default (GConfClient* client, const gchar* key,
-                      		     const gchar* def, GError** err)
-{
-	gchar *str_color = NULL;
-	GdkColor color = {0, };
-
-	gedit_debug (DEBUG_PREFS);
-
-      	g_return_val_if_fail (client != NULL, color);
-      	g_return_val_if_fail (GCONF_IS_CLIENT (client), color);  
-	g_return_val_if_fail (key != NULL, color);
-	g_return_val_if_fail (def != NULL, color);
-
-	str_color = gconf_client_get_string_with_default (client, 
-							  key, 
-							  def,
-							  NULL);
-
-	g_return_val_if_fail (str_color != NULL, color);
-
-	gdk_color_parse (str_color, &color);
-	g_free (str_color);
-	
-	return color;
-}
-
-static gboolean
-gedit_gconf_client_set_color (GConfClient   *client,
-                              const gchar   *key,
-                              GdkColor       val,
-                              GError       **err)
-{
-	gchar *str_color = NULL;
-	gboolean res;
-
-	gedit_debug (DEBUG_PREFS);
-
-	g_return_val_if_fail (client != NULL, FALSE);
-	g_return_val_if_fail (GCONF_IS_CLIENT (client), FALSE);  
-	g_return_val_if_fail (key != NULL, FALSE);
-
-	str_color = gedit_gdk_color_to_string (val);
-	g_return_val_if_fail (str_color != NULL, FALSE);
-
-	res = gconf_client_set_string (client,
-				       key,
-				       str_color,
-				       err);
-	g_free (str_color);
-
-	return res;
-}
-
-static GdkColor
-gedit_prefs_manager_get_color (const gchar* key, const gchar* def)
-{
-	GdkColor color = {0, };
-
-	gedit_debug (DEBUG_PREFS);
-
-	if (def != NULL)
-		gdk_color_parse (def, &color);
-
-	g_return_val_if_fail (gedit_prefs_manager != NULL, color);
-	g_return_val_if_fail (gedit_prefs_manager->gconf_client != NULL, color);
-
-	return gconf_client_get_color_with_default (gedit_prefs_manager->gconf_client,
-						    key,
-						    def,
-						    NULL);
-}	
-
-static void		 
-gedit_prefs_manager_set_color (const gchar* key, GdkColor value)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	g_return_if_fail (gedit_prefs_manager != NULL);
-	g_return_if_fail (gedit_prefs_manager->gconf_client != NULL);
-	g_return_if_fail (gconf_client_key_is_writable (
-				gedit_prefs_manager->gconf_client, key, NULL));
-			
-	gedit_gconf_client_set_color (gedit_prefs_manager->gconf_client, key, value, NULL);
-}
-
-
 /* Use default font */
 DEFINE_BOOL_PREF (use_default_font,
 		  GPM_USE_DEFAULT_FONT,
@@ -401,121 +314,6 @@ gedit_prefs_manager_get_system_font (void)
 					       GPM_DEFAULT_SYSTEM_FONT);
 }
 
-/* Use default colors */
-DEFINE_BOOL_PREF (use_default_colors,
-		  GPM_USE_DEFAULT_COLORS,
-		  GPM_DEFAULT_USE_DEFAULT_COLORS)
-
-
-/* Background color */	
-GdkColor
-gedit_prefs_manager_get_background_color (void)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return gedit_prefs_manager_get_color (GPM_BACKGROUND_COLOR,
-					      GPM_DEFAULT_BACKGROUND_COLOR);
-}
-
-void
-gedit_prefs_manager_set_background_color (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	gedit_prefs_manager_set_color (GPM_BACKGROUND_COLOR,
-				       color);
-}
-
-gboolean
-gedit_prefs_manager_background_color_can_set (void)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	return gedit_prefs_manager_key_is_writable (GPM_BACKGROUND_COLOR);
-}
-	
-/* Text color */	
-GdkColor
-gedit_prefs_manager_get_text_color (void)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return gedit_prefs_manager_get_color (GPM_TEXT_COLOR,
-					      GPM_DEFAULT_TEXT_COLOR);
-}
-
-void
-gedit_prefs_manager_set_text_color (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	gedit_prefs_manager_set_color (GPM_TEXT_COLOR,
-				       color);
-}
-
-gboolean
-gedit_prefs_manager_text_color_can_set (void)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	return gedit_prefs_manager_key_is_writable (GPM_TEXT_COLOR);
-}
-
-/* Selected text color */	
-GdkColor
-gedit_prefs_manager_get_selected_text_color (void)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return gedit_prefs_manager_get_color (GPM_SELECTED_TEXT_COLOR,
-					      GPM_DEFAULT_SELECTED_TEXT_COLOR);
-}
-
-void
-gedit_prefs_manager_set_selected_text_color (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	gedit_prefs_manager_set_color (GPM_SELECTED_TEXT_COLOR,
-				       color);
-}
-
-gboolean
-gedit_prefs_manager_selected_text_color_can_set (void)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	return gedit_prefs_manager_key_is_writable (GPM_SELECTED_TEXT_COLOR);
-}
-
-/* Selection color */	
-GdkColor
-gedit_prefs_manager_get_selection_color (void)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return gedit_prefs_manager_get_color (GPM_SELECTION_COLOR,
-					      GPM_DEFAULT_SELECTION_COLOR);
-}
-
-void
-gedit_prefs_manager_set_selection_color (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	gedit_prefs_manager_set_color (GPM_SELECTION_COLOR,
-				       color);
-}
-
-gboolean
-gedit_prefs_manager_selection_color_can_set (void)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	return gedit_prefs_manager_key_is_writable (GPM_SELECTION_COLOR);
-}
-
-
 /* Create backup copy */
 DEFINE_BOOL_PREF (create_backup_copy,
 		  GPM_CREATE_BACKUP_COPY,
@@ -772,8 +570,8 @@ gedit_prefs_manager_get_print_wrap_mode (void)
 void
 gedit_prefs_manager_set_print_wrap_mode (GtkWrapMode pwp)
 {
-	const gchar * str;
-	
+	const gchar *str;
+
 	gedit_debug (DEBUG_PREFS);
 
 	switch (pwp)
@@ -790,8 +588,7 @@ gedit_prefs_manager_set_print_wrap_mode (GtkWrapMode pwp)
 			str = "GTK_WRAP_CHAR";
 	}
 
-	gedit_prefs_manager_set_string (GPM_PRINT_WRAP_MODE,
-					str);
+	gedit_prefs_manager_set_string (GPM_PRINT_WRAP_MODE, str);
 }
 
 gboolean
@@ -1228,6 +1025,79 @@ DEFINE_INT_PREF (right_margin_position,
 		 GPM_RIGHT_MARGIN_POSITION,
 		 GPM_DEFAULT_RIGHT_MARGIN_POSITION)
 
+static GtkSourceSmartHomeEndType
+get_smart_home_end_from_string (const gchar *str)
+{
+	GtkSourceSmartHomeEndType res;
+
+	g_return_val_if_fail (str != NULL, GTK_SOURCE_SMART_HOME_END_AFTER);
+
+	if (strcmp (str, "DISABLED") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_DISABLED;
+	else if (strcmp (str, "BEFORE") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_BEFORE;
+	else if (strcmp (str, "ALWAYS") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_ALWAYS;
+	else
+		res = GTK_SOURCE_SMART_HOME_END_AFTER;
+
+	return res;
+}
+
+GtkSourceSmartHomeEndType
+gedit_prefs_manager_get_smart_home_end (void)
+{
+	gchar *str;
+	GtkSourceSmartHomeEndType res;
+
+	gedit_debug (DEBUG_PREFS);
+
+	str = gedit_prefs_manager_get_string (GPM_SMART_HOME_END,
+					      GPM_DEFAULT_SMART_HOME_END);
+
+	res = get_smart_home_end_from_string (str);
+
+	g_free (str);
+
+	return res;
+}
+	
+void
+gedit_prefs_manager_set_smart_home_end (GtkSourceSmartHomeEndType smart_he)
+{
+	const gchar *str;
+
+	gedit_debug (DEBUG_PREFS);
+
+	switch (smart_he)
+	{
+		case GTK_SOURCE_SMART_HOME_END_DISABLED:
+			str = "DISABLED";
+			break;
+
+		case GTK_SOURCE_SMART_HOME_END_BEFORE:
+			str = "BEFORE";
+			break;
+
+		case GTK_SOURCE_SMART_HOME_END_ALWAYS:
+			str = "ALWAYS";
+			break;
+
+		default: /* GTK_SOURCE_SMART_HOME_END_AFTER */
+			str = "AFTER";
+	}
+
+	gedit_prefs_manager_set_string (GPM_WRAP_MODE, str);
+}
+
+gboolean
+gedit_prefs_manager_smart_home_end_can_set (void)
+{
+	gedit_debug (DEBUG_PREFS);
+	
+	return gedit_prefs_manager_key_is_writable (GPM_SMART_HOME_END);
+}
+
 /* Enable syntax highlighting */
 DEFINE_BOOL_PREF (enable_syntax_highlighting,
 		  GPM_SYNTAX_HL_ENABLE,
